/*      @(#)tool_hs.h 1.1 92/07/30 SMI      */

/*
 * Copyright (c) 1983 by Sun Microsystems, Inc.
 */

/*
 * Include this header file to get all tool building related header files.
 */

#ifndef	tool_hs_defined
#define	tool_hs_defined

#include <signal.h>
#include <pixrect/pixrect_hs.h>
#include <sunwindow/window_hs.h>
#include <suntool/icon.h>
#include <suntool/walkmenu.h>
#include <suntool/tool.h>

#endif	tool_hs_defined
