/* @(#)errors.c 1.1 92/07/30 SMI */

/*
 * Copyright (c) 1989, 1991 Sun Microsystems, Inc.
 */

char DLE_none[] = "ld.so: no error";
char DLE_mode_error[] = "ld.so: invalid dlopen mode";
char DLE_bad_handle[] = "ld.so: invalid object";
char DLE_can_not_open[] = "ld.so: object could not be mapped";
char DLE_undefined[] = "ld.so: undefined symbol";
char DLE_conflict[] = "ld.so: conflicting usage of dlopen'ed dependents";
