/*	@(#)fcntl.h 1.1 92/07/30 SMI	*/
/*	@(#)fcntl.h 1.6 88/02/08 SMI	*/

#ifndef	_FCNTL_
#define	_FCNTL_

#include <sys/fcntlcom.h>

#define	O_NDELAY	_FNDELAY	/* Non-blocking I/O (4.2 style) */

#endif	/* !_FCNTL_ */
