        .data
|        .asciz  "@(#)Vfltd.s 1.1 92/07/30 Copyr 1985 Sun Micro"
        .even
        .text

|       Copyright (c) 1985 by Sun Microsystems, Inc.

#include "fpcrtdefs.h"

VECTOR(fltd,fvflti)
