!	.data
!	.asciz	"@(#)start_float.s 1.1 92/07/30 SMI"

!	Copyright (c) 1989 by Sun Microsystems, Inc.

        .seg    "text"
        .align  4
        .global start_float
start_float:
	retl
	nop				! [internal]
