#include <sys/types.h>
#define KERNEL
#include <sys/file.h>

file
./"flag"16t"type"8t"count"8t"msgcount"n{f_flag,X}{f_type,x}{f_count,d}{f_msgcount,d}n"vnode"16t"offset"16t"cred"n{f_data,X}{f_offset,X}{f_cred,X}
+,<9-1$<file
