#ifndef lint
static	char sccsid[] = "@(#)clean.c 1.1 92/07/30 SMI"; /* from System III 3.1 */
#endif

/*
	Default clean_up routine for fatal and setsig.
	User defined clean_up routines are used for removing
	temporary files, etc.
*/

clean_up()
{
}
