/*
 * @(#)trace.h 1.1 7/30/92 Copyright Sun Microsystems
 */

/******************************************************************************

  trace.h

  Author:	Anantha N. Srirama		anantha@ravi

  This file contains two defines which are used by the function 'trace' to
  determine what to prepend to a message.

  This file should be absorbed into the file 'sdrtns.h', consequently this
  files inclusion must be removed from all the affected files.

******************************************************************************/
#define	ENTER	0
#define EXIT	1

