/*	@(#)libaudio_impl.h 1.1 92/07/30 SMI	*/
/* Copyright (c) 1989 by Sun Microsystems, Inc. */

#ifndef _multimedia_libaudio_impl_h
#define	_multimedia_libaudio_impl_h

#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include "libaudio.h"
#include "archdep.h"

#endif /*!_multimedia_libaudio_impl_h*/
