#ifndef lint
static	char	sccsid[] = "@(#)dummy.c 1.1 92/07/30";
#endif lint


#include <varargs.h>
#include "menu.h"


main() {
	(void) add_confirm_obj((form *) 0, 0, (char *) 0, (char *) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0, (char *) 0);
	(void) add_finish_obj((pointer) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_form_button((form_radio *) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, 0, (int (*)()) 0, (pointer) 0);
	(void) add_form_field((form *) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, (menu_coord) 0, (char *) 0, 0, (int (*)()) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_form_noecho((form *) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, (char *) 0, 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_form_radio((form *) 0, (int (*)()) 0, (char) 0, (char *) 0, (int *) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_form_yesno((form *) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, (char *) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_menu_file((pointer) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, (menu_coord) 0, (menu_coord) 0, (menu_coord) 0, (char *) 0);
	(void) add_menu_item((menu *) 0, (int (*)()) 0, (char) 0, (char *) 0, (menu_coord) 0, (menu_coord) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0, (int (*)()) 0, (pointer) 0);
	(void) add_menu_string((pointer) 0, (char) 0, (menu_coord) 0, (menu_coord) 0, (menu_attr) 0, (char *) 0);
	(void) ckf_abspath((pointer) 0, (char *) 0);
	(void) ckf_empty((pointer) 0, (char *) 0);
	(void) ckf_ether_aton((pointer) 0, (char *) 0);
	(void) ckf_inet_addr((pointer) 0, (char *) 0);
	(void) ckf_int((pointer) 0, (char *) 0);
	(void) ckf_long((pointer) 0, (char *) 0);
	(void) ckf_netabs((pointer) 0, (char *) 0);
	(void) ckf_netpath((pointer) 0, (char *) 0);
	(void) ckf_octal((pointer) 0, (char *) 0);
	(void) ckf_relpath((pointer) 0, (char *) 0);
	(void) ckf_uint((pointer) 0, (char *) 0);
	clear_form((form *) 0);
	clear_form_button((form_button *) 0);
	clear_form_field((form_field *) 0);
	clear_form_noecho((form_noecho *) 0);
	clear_form_radio((form_radio *) 0);
	clear_form_yesno((form_yesno *) 0);
	clear_menu((menu *) 0);
	clear_menu_file((menu_file *) 0);
	clear_menu_item((menu_item *) 0);
	clear_menu_string((menu_string *) 0);
	(void) create_form((int (*)()) 0, 0, (char *) 0);
	(void) create_menu((int (*)()) 0, 0, (char *) 0);
	display_form((form *) 0);
	display_form_button((form_button *) 0);
	display_form_field((form_field *) 0);
	display_form_noecho((form_noecho *) 0);
	display_form_radio((form_radio *) 0);
	display_form_yesno((form_yesno *) 0);
	display_item_status((menu_item *) 0);
	display_menu((menu *) 0);
	display_menu_file((menu_file *) 0);
	display_menu_item((menu_item *) 0);
	display_menu_string((menu_string *) 0);
	end_menu();
	(void) find_form_button((form *) 0, (char *) 0);
	(void) find_form_field((form *) 0, (char *) 0);
	(void) find_form_noecho((form *) 0, (char *) 0);
	(void) find_form_radio((form *) 0, (char *) 0);
	(void) find_form_yesno((form *) 0, (char *) 0);
	(void) find_menu_file((pointer) 0, (char *) 0);
	(void) find_menu_item((menu *) 0, (char *) 0);
	form_help_screen();
	free_form((form *) 0);
	free_form_button((form_radio *) 0, (form_button *) 0);
	free_form_field((form *) 0, (form_field *) 0);
	free_form_map((form *) 0, (pointer) 0, (char *) 0);
	free_form_noecho((form *) 0, (form_noecho *) 0);
	free_form_radio((form *) 0, (form_radio *) 0);
	free_form_yesno((form *) 0, (form_yesno *) 0);
	free_menu((menu *) 0);
	free_menu_file((pointer) 0, (menu_file *) 0);
	free_menu_item((menu *) 0, (menu_item *) 0);
	free_menu_string((pointer) 0, (menu_string *) 0);
	(void) get_form_field((form_field *) 0);
	(void) get_form_noecho((form_noecho *) 0);
	(void) get_form_radio((form_radio *) 0);
	(void) get_form_yesno((form_yesno *) 0);
	(void) get_menu_item((menu_item *) 0);
	help_screen_off();
	help_screen_on();
	init_menu();
	(void) is_help_screen();
	(void) lex_no_ws((char *) 0, (char *) 0, (char) 0);
	menu_abort(0);
	menu_ack();
	(char) menu_ask_num((int) 0, (int) 0);
	(void) menu_alloc((unsigned int) 0);
	(void) menu_ask_yesno((int) 0, (char *) 0);
	(void) menu_cols();
	menu_flash_off();
	menu_flash_on((char *) 0);
	(void) menu_goto_obj();
	menu_help_screen();
	(void) menu_ignore_obj();
	(void) menu_lines();
	menu_log((char *) 0);
	menu_mesg((char *) 0);
	(void) menu_print((char *) 0, (va_list) 0);
	(void) menu_repeat_obj();
	menu_sig_trap(0);
	(void) menu_skip_io();
	off_form((form *) 0);
	off_form_button((form_button *) 0);
	off_form_field((form_field *) 0);
	off_form_noecho((form_noecho *) 0);
	off_form_radio((form_radio *) 0);
	off_form_shared((form *) 0, (form_yesno *) 0);
	off_form_yesno((form_yesno *) 0);
	off_menu((menu *) 0);
	off_menu_file((menu_file *) 0);
	off_menu_item((menu_item *) 0);
	off_menu_string((menu_string *) 0);
	on_form((form *) 0);
	on_form_button((form_button *) 0);
	on_form_field((form_field *) 0);
	on_form_noecho((form_noecho *) 0);
	on_form_radio((form_radio *) 0);
	on_form_shared((form *) 0, (form_yesno *) 0);
	on_form_yesno((form_yesno *) 0);
	on_menu((menu *) 0);
	on_menu_file((menu_file *) 0);
	on_menu_item((menu_item *) 0);
	on_menu_string((menu_string *) 0);
	paint_menu();
	(void) read_menu_char();
	redisplay(0);
	set_form_map((form *) 0, (pointer) 0);
	set_menu_item((menu *) 0, (menu_item *) 0);
	set_menu_log((char *) 0);
	set_menu_term((char *) 0);
	(void) use_form((form *) 0);
	(void) use_menu((menu *) 0);
	_menu_mesg((char *) 0, (va_list) 0);
} /* end main() */
