/*	@(#)bkgcodes.h 1.1 92/07/30 SMI */

/*
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

#define ACCEPT_DBLE	'A'	/* acceptance of a double */
#define CANTMOVE	'C'	/* current player can't move */
#define DOUBLEREQ	'D'	/* offer to double */
#define HUMANFIRST	'H'	/* human goes first */
#define COMPUTERFIRST	'K'	/* computer goes first */
#define DIEROLL		'L'	/* roll of the dice follows */
#define CANMOVE		'M'	/* current player is moving */
#define NODOUBLE	'N'	/* current player isn't doubling this turn */
#define QUIT		'Q'	/* bkg is done */
#define REFUSE_DBLE	'R'	/* refusal of a double */
#define ERRMSG		'^'	/* an error message follows */
