
#ifndef lint
static  char sccsid[] = "@(#)rp_recip_table.c 1.1 92/07/30 Copyr 1990 Sun Micro";
#endif

/*
 * Copyright (c) 1990 by Sun Microsystems, Inc.
 */

#include <stdio.h>
#include <sdrtns.h>     /* sdrtns.h should always be included */
#include <errmsg.h>

/**********************************************************************/
char *
rp_recip_table()
/**********************************************************************/

{

    func_name = "rp_recip_table";
    TRACE_IN

    TRACE_OUT
    return(DLXERR_NOT_IMPLEMENTED);
}
