/*
 * Copyright (c) 1980 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef lint
static char sccsid[] = "@(#)label.c 1.1 92/07/30 SMI"; /* from UCB 5.1 6/7/85 */
#endif not lint

label_(s, len)
register char *s;
long len;
{
	char buf[260];
	register char *cp, *cend;

	cp = buf;
	cend = cp + (len < 255 ? len : 255 );
	while ( cp < cend ) 
		*cp++ = *s++;
	*cp = 0;
	label( buf );
}
