/*
 * @(#)printer.h - Rev 1.1 - 7/30/92
 *
 * printer.h:  header file for printer.c.
 *
 */

/*
 * Error code for this test.
 */
#define OPEN_ERROR              6
#define WRITE_ERROR             7
#define PE_ERROR                8
#define SLCT_ERROR              9
#define ODD_ERROR               10
#define EVEN_ERROR              11
