/* 
 * @(#)sunmon.h 1.1 92/07/30 SMI
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#ifndef _mon_sunmon_h
#define _mon_sunmon_h

#ifdef sun2
#include "../sun2/sunmon.h"
#endif 

#ifdef sun3
#include "../sun3/sunmon.h"
#endif 

#ifdef sun3x
#include "../sun3x/sunmon.h"
#endif

#ifdef sun4 
#include "../sun4/sunmon.h"
#endif 

#endif /*!_mon_sunmon_h*/
