/*	@(#)curses_inc.h 1.1 92/07/30 SMI; from S5R3.1 1.2	*/

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 * Internal header file for the library
 *
 */

/* LINTLIBRARY */

#define		CURSES	/* We are internal to curses */

#include	"curses.h"
#include	"term.h"
#include	"curshdr.h"
#include	"unctrl.h"
