#include <sys/vtimes.h>

vtimes
./"utime"16t"stime"16t"idsrss"16t"ixrss"n{vm_utime,X}{vm_stime,X}{vm_idsrss,X}{vm_ixrss,X}
+/"maxrss"16t"majflt"16t"minflt"16t"nswap"n{vm_maxrss,X}{vm_majflt,X}{vm_minflt,X}{vm_nswap,X}
+/"inblk"16t"oublk"n{vm_inblk,X}{vm_oublk,X}
