/*
 * Copyright (c) 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1988 Regents of the University of California.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
static	char sccsid[] = "@(#)whoami.c 1.1 92/07/30 SMI"; /* from UCB 5.3 6/29/88 */
#endif not lint

#include <sys/types.h>
#include <pwd.h>

main()
{
	struct passwd *p, *getpwuid();
	uid_t uid;

	uid = geteuid();
	if (!(p = getpwuid((int) uid))) {
		printf("whoami: no login associated with uid %u.\n", uid);
		exit(1);
	}
	printf("%s\n", p->pw_name);
	exit(0);
	/* NOTREACHED */
}
