/*      @(#)fpu.h	1.1 7/30/92 Copyright Sun Micro     */

/* 
 *  Co-Processor Types 
 */

#define	WEITEK		0		/* Weitek family		*/
#define	TI8847		10		/* TI family 		8847	*/
#define MC68881		20		/* Motorola family 	68881	*/
#define FAB6		30		/* FAB6 			*/
#define INVALID		-1		/* Invalid device code		*/

