#ifndef lint
static	char sccsid[] = "@(#)release.c 1.1 92/07/30 SMI";
#endif

/*
 * Copyright (c) 1989 by Sun Microsystems, Inc.
 */

#ident "@(#)RELEASE libm.a 4.1, alpha 9"

void release_dummy()
{
}
