/* @(#)spif_msg.h 1.1 7/30/92 Copyright 1992 Sun Microsystems. */

/*
 * spif_msg.h -
 *
 * Message file for spiftest.c
 */

extern char	*test_usage_msg;
extern char	*start_test_msg;
extern char	*end_test_msg;
extern char	*routine_msg;
extern char	*bad_device_msg;
extern char	*probe_err_msg;
extern char	*parms_msg;
extern char	*data_msg;
extern char	*internal_test;
extern char	*lp96_test;
extern char	*lp96_msg;
extern char	*lp25_test;
extern char	*lp25_msg;
extern char	*echotty_test;
extern char	*echotty_msg;
extern char	*printer_test;
extern char	*printer_msg;
extern char	*pploop_test;
extern char	*modemloop_test;
extern char	*dataloop_test;
extern char	*open_err_msg;
extern char	*read_err_msg;
extern char	*write_err_msg;
extern char	*testing_msg;
extern char	*device_open;
extern char	*tcsets_err;
extern char	*tiocmget_err;
extern char	*tiocmset_err;
extern char	*tiocmbic_err;
extern char	*stc_regiow_cor2;
extern char	*stc_regiow_ccr;
extern char	*stc_regior_ccr;
extern char	*stc_ppcregw;
extern char	*stc_ppcregr;
extern char	*stc_dcontrol;
extern char	*stc_sdefaults;
extern char	*stc_gdefaults;
extern char	*stc_gppc;
extern char	*offline_err;
extern char	*paper_out_err;
extern char	*busy_err;
extern char	*pp_err;
extern char	*compare_err_msg;
extern char	*mismatch_err_msg;
extern char	*dsr_clear_err;
extern char	*dsr_set_err;
extern char	*internal_test_err;
extern char	*data_test_err;
extern char	*pploop_test_err;
extern char	*modemloop_test_err;
extern char	*lp_err_msg;
extern char	*tty_err_msg;
extern char	*end_echotty;
extern char	*timeout_err;
