/*	@(#)vmparam.h 1.1 92/07/30 SMI; from UCB 4.13 82/12/17	*/

/*
 * Machine dependent constants
 */

#ifndef _sys_vmparam_h
#define _sys_vmparam_h

#include <machine/vmparam.h>

#endif /*!_sys_vmparam_h*/
