#ifndef lint
static  char sccsid[] = "@(#)stubs.c 1.1 92/07/30";
#endif

#include "reg.h"

struct allregs regsave;

int pagesused;
int lastpg;
int lastpm;

scbsync(){}
startup(){}
main(){}
fault(){}
cmd(){}
level14(){}
level15(){}
