/*	@(#)rcv.h 1.1 92/07/30 SMI; from S5R2 1.1	*/

/*
 * mailx -- a modified version of a University of California at Berkeley
 *	mail program
 *
 * This file is included by normal files which want both
 * globals and declarations.
 */

#include "def.h"
#include "glob.h"
