#include <sys/types.h>
#include <sys/socket.h>
#include <sys/socketvar.h>

sockbuf
./"cc"8t"hiwat"8t"mbcnt"8t"mbmax"8t"lowat"8t"timeo"n{sb_cc,d}{sb_hiwat,d}{sb_mbcnt,d}{sb_mbmax,d}{sb_lowat,d}{sb_timeo,d}n"mb"16t"sel"16t"flags"n{sb_mb,X}{sb_sel,X}{sb_flags,x}
