/*	@(#)timeout.h 1.1 92/07/30 SMI; from S5R3.1 1.7	*/

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

/*
 *	UNIX shell
 */

#define TIMEOUT 	0	/* seconds elapsing before log-off (normal) */
#define MAILCHECK	"600"	/* 10 minutes */
