/*
 * @(#)pmem_msg.h - Rev 1.1 - 7/30/92
 *
 * pmem_msg.h:  header file for pmem.c.
 *
 */

extern char *test_usage_msg;
extern char *kvmopen_err_msg;
extern char *kvmnlist_err_msg;
extern char *namelist_err_msg;
extern char *read_err_msg;
extern char *no_memory_msg;
extern char *test_msg;
extern char *memory_size_msg;
extern char *read_end_msg;
extern char *open_err_msg;
extern char *valloc_err_msg;
extern char *memlist_err_msg;
extern char *found_err_msg;
extern char *pmlist_err_msg;
extern char *read_mem_msg;
extern char *mmap_err_msg;
extern char *size_err_msg;
extern char *routine_msg;
