/*
 * @(#)mono_msg.h - Rev 1.1 - 7/30/92
 *
 * mono_msg.h:  header file for mono.c.
 *
 */

extern char *test_usage_msg;
extern char *e_usage_msg;
extern char *e_ioctl_msg;
extern char *e_openfb_msg;
extern char *v_start_msg;
extern char *e_pr_open_msg;
extern char *v_fbsize_msg;
extern char *v_completed_msg;
extern char *e_open_msg;
extern char *e_mmap_msg;
extern char *v_fbaddr_msg;
extern char *e_const_msg;
extern char *e_addr_msg;
extern char *e_laddr_msg;
extern char *e_rand_msg;
extern char *e_stripe_msg;
extern char *routine_msg;
