/*      @(#)priv_prot.h 1.1 92/07/30 SMI                              */

/*
 * This file consists of all rpc information for lock manager
 * interface to status monitor
 */

#define PRIV_PROG 100021
#define PRIV_VERS 2
#define PRIV_CRASH 1
#define PRIV_RECOVERY 2

