/*
 * @(#)tapetest_msg.h - Rev 1.1 - 7/30/92
 *
 * tapetest_msg.h:  header file for tapetest.c.
 *
 */

extern char *devname_err_msg;
extern char *devfile_err_msg;
extern char *bad_devfile_msg;
extern char *read_only_msg;
extern char *sharing_msg;
extern char *rewind_msg;
extern char *retension_msg;
extern char *erase_msg;
extern char *start_test_msg;
extern char *write_end_msg;
extern char *read_end_msg;
extern char *write_blk_msg;
extern char *read_blk_msg;
extern char *rewind_after_msg;
extern char *switch_2nd_msg;
extern char *switch_next_msg;
extern char *reconnect_msg;
extern char *sleep_msg;
extern char *file_test_msg;
extern char *hit_eot_msg;
extern char *open_err_msg;
extern char *close_err_msg;
extern char *tape_status_msg;
extern char *test_usage_msg;
extern char *compare_err_msg;
extern char *scsi_io_msg;
extern char *nonscsi_io_msg;
extern char *routine_msg;

extern char *recon_start_msg;
extern char *recon_stop_msg;
extern char *tape_name_msg;
extern char *disk_name_msg;
extern char *device_err_msg;
extern char *environ_err_msg;
extern char *reten_err_msg;
extern char *file_err_msg;
extern char *retry_msg;
extern char *connect_err_msg;
extern char *spurious_msg;
extern char *reten_err_msg;
