#include <sys/types.h>
#include <sys/stream.h>

datab
./"freep"16t"base"16t"lim"16tn{db_freep,X}{db_base,X}{db_lim,X}n"ref"8t"type"8t"class"8t"size"n{db_ref,b}{db_type,b}{db_class,b}{db_size,D}n"cache"n{db_cache,xxxxxxxx}{END}
