
/*
 * @(#)cpu.map.h 1.1 92/07/30
 * Copyright (c) 1985 by Sun Microsystems, Inc.
 */

#ifdef sun2
#include "../../mon/sun2/cpu.map.h"
#endif 

#ifdef sun3
#include "../../mon/sun3/cpu.map.h"
#endif 

#ifdef sun3x
#include "../../mon/sun3x/cpu.map.h"
#endif

#ifdef sun4
#include "../../mon/sun4/cpu.map.h"
#endif 
