/*
 * @(#)fstest_msg.h - Rev 1.1 - 7/30/92
 *
 * fstest_msg.h:  header file for fstest.c.
 *
 */

extern char *test_usage_msg;
extern char *reread_err_msg;
extern char *open_file_msg;
extern char *mount_err_msg;
extern char *auto_mnt_err_msg;
extern char *unmnt_err_msg;
extern char *add_err_msg;
extern char *free_err_msg;
extern char *transfer_err_msg;
extern char *flag_err_msg;
extern char *pmount_err_msg;
extern char *pwrite_err_msg;
extern char *blocks_err_msg;
extern char *nofs_err_msg;
extern char *opendev_err_msg;
extern char *routine_msg;
extern char *debug_info_msg;
extern char *display1_msg;
extern char *display2_msg;
extern char *data_msg;
extern char *continue_msg;
extern char *recompare_msg;
extern char *recompare2_msg;
extern char *good_recomp_msg;
extern char *auto_err_msg;
