/*	@(#)headers.h 1.1 92/07/30 SMI	*/

/*
 * Copyright (c) 1987 Sun Microsystems, Inc.
 */

#include "sysdep.h"
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/dir.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/syscall.h>
#include <sys/user.h>
#include <strings.h>
#include <stdio.h>
