/******************************************************************************
	@(#)fpa3x_msg.h - Rev 1.1 - 7/30/92
	Copyright (c) 1988 by Sun Microsystems, Inc.
	Deyoung Hong

	FPA-3X message file.
******************************************************************************/


extern char


/* Test messages */
*tst_context ,
*tst_status ,
*tst_open ,
*tst_close ,
*tst_spmath ,
*tst_dpmath ,
*tst_dptrig ,
*tst_reg ,
*tst_datareg ,
*tst_shadow ,
*tst_nack ,
*tst_simpleins ,
*tst_pointer ,
*tst_lock ,
*tst_jumpcond ,
*tst_tipath ,
*tst_tiop ,
*tst_tistat ,
*tst_timing ,
*tst_linpack ,


/* Error messages */
*er_opendev ,
*er_closedev ,
*er_nofpa ,
*er_no68881 ,
*er_readfpa ,
*er_writefpa ,
*er_spmath ,
*er_dpmath ,
*er_dptrig ,
*er_reg ,
*er_wstatus ,
*er_datareg ,
*er_shadow ,
*er_simpleins ,
*er_nonack ,
*er_nack ,
*er_pointer ,
*er_nolock ,
*er_lock ,
*er_lockierr ,
*er_jumpcond ,
*er_tipath ,
*er_tiop ,
*er_tistat ,
*er_timing ,
*er_linpack ,
*er_usage ;


