set append dot
if t
	# Mail-only defaults here
else
	# mailtool-only defaults here
	#
	# this should probably not be mailtool-only, but for
	# compatibility we only put it here.
	set hold
	# following so hierarchical menu is the default
	set LISTER="ls -LF"
	# default is to allow editing of message windows
	set editmessagewindow
endif
