/*
 * Copyright (C) 1989 Sun Microsystems Inc.
 */

#ifndef lint
/*	@(#)sh.tconst.h 1.1 92/07/30 SMI.	*/
#endif

/*
 * C shell
 */


extern tchar S_0[];
extern tchar S_1[];
extern tchar S_ANDAND[];
extern tchar S_AND[];
extern tchar S_AST[];
extern tchar S_AT[];
extern tchar S_BARBAR[];
extern tchar S_BAR[];
extern tchar S_BRABRA[];
extern tchar S_BRAPPPBRA[];
extern tchar S_COLON[];
extern tchar S_DASHl[];
extern tchar S_DELIM[];
extern tchar S_DOTDOTSLA[];
extern tchar S_DOTSLA[];
extern tchar S_DOT[];
extern tchar S_EQ[];
extern tchar S_EXAS[];
extern tchar S_HAT[];
extern tchar S_HOME[];
extern tchar S_IOT[];
extern tchar S_LBRASP[];
extern tchar S_LBRA[];
extern tchar S_LESLES[];
extern tchar S_LPAR[];
extern tchar S_MINUS[];
extern tchar S_MINl[];
extern tchar S_NAMI[];
extern tchar S_NDOThistory[];
extern tchar S_OTHERSH[];
extern tchar S_PARCENTMINUS[];
extern tchar S_PARCENTPARCENT[];
extern tchar S_PARCENTPLUS[];
extern tchar S_PARCENTSHARP[];
extern tchar S_PATH[];
extern tchar S_PERSENTSP[];
extern tchar S_PWD[];
extern tchar S_PjobAND[];
extern tchar S_Pjob[];
extern tchar S_QPPPQ[];
extern tchar S_RBRA[];
extern tchar S_RPAR[];
extern tchar S_SEMICOLONSP[];
extern tchar S_SHARPSP[];
extern tchar S_SHELLPATH[];
extern tchar S_SLADOTcshrc[];
extern tchar S_SLADOThistory[];
extern tchar S_SLADOTlogin[];
extern tchar S_SLADOTlogout[];
extern tchar S_SLASH[];
extern tchar S_SPANDANDSP[];
extern tchar S_SPBARBARSP[];
extern tchar S_SPBARSP[];
extern tchar S_SPGTRGTRSP[];
extern tchar S_SPGTR[];
extern tchar S_SPLESLESSP[];
extern tchar S_SPLESSP[];
extern tchar S_SPPPP[];
extern tchar S_SPRBRA[];
extern tchar S_SP[];
extern tchar S_TERM[];
extern tchar S_TOPBIT[];
extern tchar S_USAGEFORMAT[];
extern tchar S_USER[];
extern tchar S_[];
extern tchar S_alias[];
extern tchar S_alloc[];
extern tchar S_aout[];
extern tchar S_argv[];
extern tchar S_bg[];
extern tchar S_bin[];
extern tchar S_break[];
extern tchar S_breaksw[];
extern tchar S_bye[];
extern tchar S_case[];
extern tchar S_cd[];
extern tchar S_cdpath[];
extern tchar S_chdir[];
extern tchar S_child[];
extern tchar S_continue[];
extern tchar S_coredumpsize[];
extern tchar S_cputime[];
extern tchar S_csh[];
extern tchar S_cwd[];
extern tchar S_datasize[];
extern tchar S_default[];
extern tchar S_descriptors[];
extern tchar S_dirs[];
extern tchar S_echo[];
extern tchar S_else[];
extern tchar S_end[];
extern tchar S_endif[];
extern tchar S_endsw[];
extern tchar S_erwxfdzo[];
extern tchar S_eval[];
extern tchar S_exec[];
extern tchar S_exit[];
extern tchar S_fg[];
extern tchar S_fignore[];
extern tchar S_filec[];
extern tchar S_filesize[];
extern tchar S_foreach[];
extern tchar S_gd[];
extern tchar S_glob[];
extern tchar S_goto[];
extern tchar S_h[];
extern tchar S_hardpaths[];
extern tchar S_hashstat[];
extern tchar S_histchars[];
extern tchar S_history[];
extern tchar S_home[];
extern tchar S_hours[];
extern tchar S_htrqxe[];
extern tchar S_if[];
extern tchar S_ignoreeof[];
extern tchar S_jobs[];
extern tchar S_kbytes[];
extern tchar S_kill[];
extern tchar S_label[];
extern tchar S_limit[];
extern tchar S_login[];
extern tchar S_logout[];
extern tchar S_mail[];
extern tchar S_megabytes[];
extern tchar S_memorysize[];
extern tchar S_minutes[];
extern tchar S_n[];
extern tchar S_newgrp[];
extern tchar S_nice[];
extern tchar S_nobeep[];
extern tchar S_noclobber[];
extern tchar S_noglob[];
extern tchar S_nohup[];
extern tchar S_nonomatch[];
extern tchar S_notify[];
extern tchar S_onintr[];
extern tchar S_path[];
extern tchar S_popd[];
extern tchar S_prompt[];
extern tchar S_pushd[];
extern tchar S_rd[];
extern tchar S_rehash[];
extern tchar S_repeat[];
extern tchar S_savehist[];
extern tchar S_seconds[];
extern tchar S_set[];
extern tchar S_setenv[];
extern tchar S_shell[];
extern tchar S_shift[];
extern tchar S_source[];
extern tchar S_stacksize[];
extern tchar S_status[];
extern tchar S_stop[];
extern tchar S_suspend[];
extern tchar S_switch[];
extern tchar S_term[];
extern tchar S_then[];
extern tchar S_time[];
extern tchar S_tmpshell[];
extern tchar S_umask[];
extern tchar S_unalias[];
extern tchar S_unhash[];
extern tchar S_unlimit[];
extern tchar S_unlimited[];
extern tchar S_unset[];
extern tchar S_unsetenv[];
extern tchar S_user[];
extern tchar S_usrbin[];
extern tchar S_usrucb[];
extern tchar S_verbose[];
extern tchar S_wait[];
extern tchar S_while[];
