/*
 * Copyright (c) 1986, 1987, 1988, 1989 by Sun Microsystems, Inc.
 * Permission to use, copy, modify, and distribute this software for any
 * purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both that copyright
 * notice and this permission notice are retained, and that the name
 * of Sun Microsystems, Inc., not be used in advertising or publicity
 * pertaining to this software without specific, written prior permission.
 * Sun Microsystems, Inc., makes no representations about the suitability
 * of this software or the interface defined in this software for any
 * purpose. It is provided "as is" without express or implied warranty.
 */
#ifndef lint
static char sccsid[] = "@(#)credelsegpas.c 1.1 92/07/30 Copyr 1983 Sun Micro";
#endif

/*
 Copyright (c) 1983 by Sun Microsystems, Inc.
*/

int close_retained_segment();
int close_temporary_segment();
int create_retained_segment();
int create_temporary_segment();
int set_image_transformation_type();

int closeretainseg(dummy)
int dummy;
	{
	return(close_retained_segment());
	}

int closetempseg(dummy)
int dummy;
	{
	return(close_temporary_segment());
	}

int createretainseg(segname)
int segname;
	{
	return(create_retained_segment(segname));
	}

int createtempseg(dummy)
int dummy;
	{
	return(create_temporary_segment());
	}

int setimgxformtype(segtype)
int segtype;
	{
	return(set_image_transformation_type(segtype));
	}
