/*
 * @(#)sas_simdef.h 1.1 92/07/30
 * Copyright (c) 1988 Sun Microsystems, Inc.
 */

/*
 * Definitions for the SAS simulated disk.
 */

#define SIMD_RDEV	"sasfs/root"
#define SIMD_RSIZE	12288
#define SIMD_SDEV	"sasfs/swap"
#define SIMD_SSIZE	12288
