/*	@(#)seln.h 1.1 92/07/30	*/

#ifndef suntool_seln_DEFINED
#define suntool_seln_DEFINED

/*
 *      Copyright (c) 1986 by Sun Microsystems, Inc.
 */

#include <suntool/selection_svc.h>
#include <suntool/selection_attributes.h>

#endif suntool_seln_DEFINED
