#include <sys/types.h>
#include <sys/stream.h>

stdata
./"wrq"16t"iocblk"16t"vnode"16t"strtab"n{sd_wrq,X}{sd_iocblk,X}{sd_vnode,X}{sd_strtab,X}n"flag"16t"iocid"16t"iocwait"8t"pgrp"8t"sigio"n{sd_flag,X}{sd_iocid,D}{sd_iocwait,d}{sd_pgrp,d}{sd_sigio,d}n"wroff"8t"error"8t"iocwait"8t"pushcnt"n{sd_wroff,u}{sd_error,b}{sd_iocwait,d}{sd_pushcnt,D}n"siglist"16t"pollist"16t"sigflags"16t"pollflags"n{sd_siglist,X}{sd_pollist,X}{sd_sigflags,X}{sd_pollflags,X}n"selr"16t"selw"16t"sele"n{sd_selr,X}{sd_selw,X}{sd_sele,X}
