/*	@(#)tiuser.h 1.1 92/07/30 SMI ; from S5R3 1.1 */

/*	Copyright (c) 1984 AT&T	*/
/*	  All Rights Reserved  	*/

/*	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF AT&T	*/
/*	The copyright notice above does not evidence any   	*/
/*	actual or intended publication of such source code.	*/

#ifndef _tiuser_h
#define _tiuser_h

#ident	"@(#)head:tiuser.h	1.1"

/*
 * TLI user interface definitions.
 */

#include <nettli/tiuser.h>

#endif /*!_tiuser_h*/
