#ifndef lint
static	char sccsid[] = "@(#)init.c 1.1 92/07/30 SMI"; /* from UCB 4.1 82/05/11 */
#endif

#include <sgtty.h>

/*
 * variable initialization.
 */

				/* name of executable object programs */
char	EXEC[] = "/usr/games/backgammon";
char	TEACH[] = "/usr/games/teachgammon";

int	pnum	= 2;		/* color of player:
					-1 = white
					 1 = red
					 0 = both
					 2 = not yet init'ed */
int	acnt	= 0;		/* length of args */
int	aflag	= 1;		/* flag to ask for rules or instructions */
int	bflag	= 0;		/* flag for automatic board printing */
int	cflag	= 0;		/* case conversion flag */
int	hflag	= 1;		/* flag for cleaning screen */
int	mflag	= 0;		/* backgammon flag */
int	raflag	= 0;		/* 'roll again' flag for recovered game */
int	rflag	= 0;		/* recovered game flag */
int	tflag	= 0;		/* cursor addressing flag */
int	iroll	= 0;		/* special flag for inputting rolls */
int	rfl	= 0;

char	*color[] = {"White","Red","white","red"};
