#ifndef lint
static	char sccsid[] = "@(#)dtopnub.c 1.1 92/07/30 Copyr 1983 Sun Micro";
#endif

/*
 * Copyright (c) 1983 by Sun Microsystems, Inc.
 */

/*
 * Must have some source file associated with a psuedo-device in order
 * to generate a dtop.h file with config.
 */

