#ifndef lint
static  char sccsid[] = "@(#)pp_test_msg.c 1.1 92/07/30 Copyright Sun Micro";
#endif

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

char    *no_open_pp      = "Unable to open Parallel Port device.";   
char    *no_save_ctl_reg = "Unable to save contents of Parallel Port Control Register."; 
char    *no_rest_ctl_reg = "Unable to restore contents of Parallel Port Control Register.";
char    *no_wr_ctl_reg   = "Unable to write to Parallel Port Control Register."; 
char    *no_rd_ctl_reg   = "Unable to read Parallel Port Control Register.";
char    *test_err_msg    = "Data Written did not match Data Read."; 
