#ifndef lint
static char     sccsid[] = "@(#)ieee_std.c 1.1 92/07/30 SMI";
#endif

/*
 * Copyright (c) 1988 by Sun Microsystems, Inc.
 */

/* By default the following C and Fortran functions do... nothing.  */

void
standard_arithmetic()
{
}

void
standard_arithmetic_()
{
}

void
nonstandard_arithmetic()
{
}

void
nonstandard_arithmetic_()
{
}

int
_is_nonstandard()
{
	return 0;
}
