SunDefaults_Version 2
;	@(#)Scrollbar.d	1.1	92/07/30 SMI
/Scrollbar	""
//Horizontal_bar_placement	"North"
	$Enumeration ""
	$Help	"Specifies placement of horizontal scrollbar (when there is one)."
	South	""
	South/$Help	"Horizontal scrollbar will appear at the bottom of the subwindow."
	North	""
	North/$Help	"Horizontal scrollbar will appear at the top of the subwindow."

//Vertical_bar_placement	"West"
	$Enumeration ""
	$Help	"Specifies placement of vertical scrollbar (when there is one)."
	West	""
	West/$Help	"Vertical scrollbar will appear at the left of the subwindow."
	East	""
	East/$Help	"Horizontal scrollbar will appear at the right of the subwindow."

//Repeat_time	"10"
	$Help	"Time, in tenths of a second, to start repeat scrolling. Use 0 to disable."

//End_point_area	"6"
	$Help	"Region, in pixels, for thumbing to either end of view."

//Bar_color	"Grey"
	$Enumeration ""
	$Help	"\"Color\" of scrollbar."
	White	""
	White/$Help	"The scrollbar will be white."
	Grey	""
	Grey/$Help	"The scrollbar will be grey."

//Bar_display_level	"Always"
	$Enumeration ""
	$Help	"Specifies under what condition to display the scrollbar."
	Always	""
	Always/$Help	"Always display the scrollbar."
	Active	""
	Active/$Help	"Display the scrollbar only when the cursor is in it."
	Never	""
	Never/$Help	"Never display the scrollbar."

//Thickness	"14"
	$Help	"The scrollbar width in pixels."

//Border	"True"
	$Enumeration ""
	$Help	"If true, display a border around the scrollbar."
	False	""
	False/$Help	"Do not display a border around the scrollbar."
	True	""
	True/$Help	"Display a border around the scrollbar."

//Bubble_margin	"0"
	$Help	"Thickness in pixels of empty margin around scroll-bubble."

//Bubble_color	"Grey"
	$Enumeration ""
	$Help	"Color of scroll-bubble."
	Black	""
	Black/$Help	"The bubble will be black."
	Grey	""
	Grey/$Help	"The bubble will be grey."

//Bubble_display_level	"Always"
	$Enumeration ""
	$Help	"Specifies under what condition to display the scrollbar bubble."
	Always	""
	Always/$Help	"Always display the bubble."
	Active	""
	Active/$Help	"Display the bubble only when the cursor is in it."
	Never	""
	Never/$Help	"Never display the bubble."

//Page_buttons	"True"
	$Enumeration ""
	$Help	"Enables/disables displaying page buttons for scrolling one line/page at a time."
	False	""
	False/$Help	"Do NOT display page buttons."
	True	""
	True/$Help	"Display page buttons."

//Page_button_length	"15"
	$Help	"The size in pixels of page buttons."

//Mess2/$Message	"You can also run the program scrolldefaults to try out different settings of the above"
//Mess3/$Message	"parameters interactively, and immediately see how the resulting scrollbars would look."
//Mess4/$Message	"When you exit scrolldefaults, the settings you selected will become your new defaults."

//$Setup_program "scrolldefaults"
	$Help	"The name of a setup program which can show the effect of these parameters."
