/*      @(#)sundiag_rpcnums.h 1.1 92/07/30 SMI          */
 
/* 
 * Copyright (c) 1988 by Sun Microsystems, Inc. 
 */

#define SUNDIAG_PROG    0x20000002
#define SUNDIAG_VERS    1
#define SD_REXPROG	0x20000001
#define SD_REXVERS	1

#define INFO_MSG        1
#define WARNING_MSG     2
#define FATAL_MSG       3
#define ERROR_MSG       4
#define DEFAULT_MSG     5
#define LOGFILE_MSG     6
#define PROBE_DEVS     	7
#define	CONSOLE_MSG	8
