#ifndef lint
static char     sccsid[] = "@(#)fp_convert.c 1.1 92/07/30 SMI";
#endif

/*
 * Copyright (c) 1987 by Sun Microsystems, Inc. 
 */

int 
_fp_convert_double_to_int(x)
	double          x;
{
	return (int) x;
}
