
/*
 *  named
 *
 *  Copyright (c) 1986 Regents of the University of California.
 *  All rights reserved.  The Berkeley software License Agreement
 *  specifies the terms and conditions for redistribution.
 */



#ifndef lint
static char sccsid[] = "@(#)version.c 1.1 92/07/30 SMI"; /* from UCB 4.8 2/28/88 */
#endif

char	Version[] = "SMI 4.1";

