/* Format_version=1, Width=64, Height=64, Depth=1, Valid_bits_per_item=16
 */
	0xFFFF,0xFFFF,0xFFFF,0xFFFF,
	0x8000,0x0000,0x0000,0x0001,
	0x8000,0x0000,0x0000,0x0001,
	0x8000,0xFFFF,0xFFFF,0xFFF9,
	0x8000,0xFFFF,0xFFFF,0xFFF9,
	0x8000,0xFFFF,0xFFFF,0xFFF9,
	0x8001,0xFFFF,0xFFFF,0xFFF9,
	0x8001,0xFFFF,0xFFFF,0xFFF9,
	0x8001,0xFFFF,0xFFFF,0xFFF9,
	0x8001,0xFFFF,0xFFFF,0xFFF9,
	0x8001,0xFFFF,0xFFFF,0xFFF9,
	0x8003,0xFFFF,0xFFFF,0xFFF9,
	0x8003,0xFFFF,0xFFFF,0xFFF9,
	0x8003,0xFFFF,0xFFFF,0xFFF9,
	0x8003,0xFFFF,0xFFFF,0xFFF9,
	0x8007,0xFFFF,0xFFFF,0xFFF9,
	0x8007,0xFFFF,0xFFFF,0xFFF9,
	0x8007,0xFFFF,0xFFFF,0xFFF9,
	0x8007,0xFFFF,0xFFFF,0xFFF9,
	0x800F,0xFFFF,0xFFFF,0xFFF9,
	0x800F,0xFFFF,0xFFFF,0xFFF9,
	0x800F,0xFFFF,0xFFFF,0xFFF9,
	0x800F,0xFFFF,0xFFFF,0xFFF9,
	0x801F,0xFFFF,0xFFFF,0xFFF9,
	0x801F,0xFFFF,0xFFFF,0xFFF9,
	0x801F,0xFFFF,0xFFFF,0xFFF9,
	0x801F,0xFFFF,0xFFFF,0xFFF9,
	0x801F,0xFFFF,0xFFFF,0xFFF9,
	0x803F,0xFFFF,0xFFFF,0xFFF9,
	0x803F,0xFFFF,0xFFFF,0xFFF9,
	0x803F,0xFFFF,0xFFFF,0xFFF9,
	0x803F,0xFFFF,0xFFFF,0xFFF1,
	0x807F,0xFFFF,0xFFFF,0xFFF1,
	0x807F,0xFFFF,0xFFFF,0xFFF1,
	0x807F,0xFFFF,0xFFFF,0xFFE1,
	0x807F,0xFFFF,0xFFFF,0xFFE1,
	0x80FF,0xFFFF,0xFFFF,0xFFE1,
	0x80FF,0xFFFF,0xFFFF,0xFFC1,
	0x80FF,0xFFFF,0xFFFF,0xFFC1,
	0x80FF,0xFFFF,0xFFFF,0xFF81,
	0x80FF,0xFFFF,0xFFFF,0xFF81,
	0x81FF,0xFFFF,0xFFFF,0xFF81,
	0x81FF,0xFFFF,0xFFFF,0xFF01,
	0x81FF,0xFFFF,0xFFFF,0xFF01,
	0x81FF,0xFFFF,0xFFFF,0xFF01,
	0x83FF,0xFFFF,0xFFFF,0xFE01,
	0x83FF,0xFFFF,0xFFFF,0xFE01,
	0x83FF,0xFFFF,0xFFFF,0xFE01,
	0x83FF,0xFFFF,0xFFFF,0xFC01,
	0x87FF,0xFFFF,0xFFFF,0xFC01,
	0x87FF,0xFFFF,0xFFFF,0xFC01,
	0x87FF,0xFFFF,0xFFFF,0xF801,
	0x8FFF,0xFFFF,0xFFFF,0xF801,
	0x8FFF,0xFFFF,0xFFFF,0xF001,
	0x8FFF,0xFFFF,0xFFFF,0xF001,
	0x8FFF,0xFFFF,0xFFFF,0xF001,
	0x8FFF,0xFFFF,0xFFFF,0xE001,
	0x8FFF,0xFFFF,0xFFFF,0xE001,
	0x8FFF,0xFFFF,0xFFFF,0xE001,
	0x8FFF,0xFFFF,0xFFFF,0xC001,
	0x8000,0x0000,0x0000,0x0001,
	0x8000,0x0000,0x0000,0x0001,
	0x8000,0x0000,0x0000,0x0001,
	0xFFFF,0xFFFF,0xFFFF,0xFFFF
